@@ -4,11 +4,14 @@ from .defs import Layer
 import re
 
 def references(board: pcbnew.BOARD, show: bool, pattern: str,
-               allowedLayers: Set[Layer] = set(Layer.all())) -> None:
+               allowedLayers: Set[Layer] = set(Layer.all()),
+               isSelectedItemOnly: bool = False) -> None:
     """
     Show or hide references in a footprint.
     """
     for footprint in board.GetFootprints():
+        if isSelectedItemOnly and not footprint.IsSelected():
+            continue
         if re.match(pattern, footprint.GetReference()) and footprint.Reference().GetLayer() in allowedLayers:
             footprint.Reference().SetVisible(show)
         for x in footprint.GraphicalItems():
@@ -19,11 +22,14 @@ def references(board: pcbnew.BOARD, show: bool, pattern: str,
 
 
 def values(board: pcbnew.BOARD, show: bool, pattern: str,
-           allowedLayers: Set[Layer] = set(Layer.all())) -> None:
+           allowedLayers: Set[Layer] = set(Layer.all()),
+           isSelectedItemOnly: bool = False) -> None:
     """
     Show or hide values in a footprint.
     """
     for footprint in board.GetFootprints():
+        if isSelectedItemOnly and not footprint.IsSelected():
+            continue
         if re.match(pattern, footprint.GetReference()) and footprint.Value().GetLayer() in allowedLayers:
             footprint.Value().SetVisible(show)
         for x in footprint.GraphicalItems():
