@@ -1,5 +1,4 @@
 from pcbnewTransition import pcbnew
-from pcbnewTransition.pcbnew import VECTOR2I
 import numpy as np
 import json
 from collections import OrderedDict
@@ -31,8 +30,8 @@ def addBottomCounterpart(board, item):
 def addRoundedCorner(board, center, start, end, thickness):
     corner = pcbnew.PCB_SHAPE()
     corner.SetShape(STROKE_T.S_ARC)
-    corner.SetCenter(VECTOR2I(center[0], center[1]))
-    corner.SetStart(VECTOR2I(start[0], start[1]))
+    corner.SetCenter(toKiCADPoint((center[0], center[1])))
+    corner.SetStart(toKiCADPoint((start[0], start[1])))
 
     if np.cross(start - center, end - center) > 0:
         corner.SetArcAngleAndEnd(fromDegrees(90), True)
@@ -46,8 +45,8 @@ def addRoundedCorner(board, center, start, end, thickness):
 def addLine(board, start, end, thickness):
     line = pcbnew.PCB_SHAPE()
     line.SetShape(STROKE_T.S_SEGMENT)
-    line.SetStart(VECTOR2I(start[0], start[1]))
-    line.SetEnd(VECTOR2I(end[0], end[1]))
+    line.SetStart(toKiCADPoint((start[0], start[1])))
+    line.SetEnd(toKiCADPoint((end[0], end[1])))
     line.SetWidth(thickness)
     line.SetLayer(Layer.F_Paste)
     board.Add(line)
@@ -60,9 +59,9 @@ def addBite(board, origin, direction, normal, thickness):
     """
     direction = normalize(direction) * thickness
     normal = normalize(normal) * thickness
-    center = VECTOR2I(origin[0], origin[1]) + VECTOR2I(normal[0], normal[1])
+    center = toKiCADPoint((origin[0], origin[1])) + toKiCADPoint((normal[0], normal[1]))
     start = origin
-    end = center + VECTOR2I(direction[0], direction[1])
+    end = center + toKiCADPoint((direction[0], direction[1]))
     # addLine(board, end, end + normal / 2, thickness)
     addRoundedCorner(board, center, start, end, thickness)
 
@@ -81,10 +80,10 @@ def addFrame(board, rect, bridgeWidth, bridgeSpacing, clearance):
     R=fromMm(1)
 
     corners = [
-        (tl(rect), VECTOR2I(R, 0), VECTOR2I(0, R)), # TL
-        (tr(rect), VECTOR2I(0, R), VECTOR2I(-R, 0)), # TR
-        (br(rect), VECTOR2I(-R, 0), VECTOR2I(0, -R)), # BR
-        (bl(rect), VECTOR2I(0, -R), VECTOR2I(R, 0)) # BL
+        (tl(rect), toKiCADPoint((R, 0)), toKiCADPoint((0, R))), # TL
+        (tr(rect), toKiCADPoint((0, R)), toKiCADPoint((-R, 0))), # TR
+        (br(rect), toKiCADPoint((-R, 0)), toKiCADPoint((0, -R))), # BR
+        (bl(rect), toKiCADPoint((0, -R)), toKiCADPoint((R, 0))) # BL
     ]
     for c, sOffset, eOffset in corners:
         addRoundedCorner(board, c + sOffset + eOffset, c + sOffset, c + eOffset, clearance)
@@ -95,16 +94,16 @@ def addFrame(board, rect, bridgeWidth, bridgeSpacing, clearance):
         end = start + cutLength
 
         y1, y2 = rect.GetY(), rect.GetY() + rect.GetHeight()
-        addLine(board, VECTOR2I(start, y1), VECTOR2I(end, y1), clearance)
+        addLine(board, toKiCADPoint((start, y1)), toKiCADPoint((end, y1)), clearance)
         if i != 0:
-            addBite(board, VECTOR2I(start, y1), VECTOR2I(-1, 0), VECTOR2I(0, 1), clearance)
+            addBite(board, toKiCADPoint((start, y1)), toKiCADPoint((-1, 0)), toKiCADPoint((0, 1)), clearance)
         if i != count - 1:
-            addBite(board, VECTOR2I(end, y1), VECTOR2I(1, 0), VECTOR2I(0, 1), clearance)
-        addLine(board, VECTOR2I(start, y2), VECTOR2I(end, y2), clearance)
+            addBite(board, toKiCADPoint((end, y1)), toKiCADPoint((1, 0)), toKiCADPoint((0, 1)), clearance)
+        addLine(board, toKiCADPoint((start, y2)), toKiCADPoint((end, y2)), clearance)
         if i != 0:
-            addBite(board, VECTOR2I(start, y2), VECTOR2I(-1, 0), VECTOR2I(0, -1), clearance)
+            addBite(board, toKiCADPoint((start, y2)), toKiCADPoint((-1, 0)), toKiCADPoint((0, -1)), clearance)
         if i != count - 1:
-            addBite(board, VECTOR2I(end, y2), VECTOR2I(1, 0), VECTOR2I(0, -1), clearance)
+            addBite(board, toKiCADPoint((end, y2)), toKiCADPoint((1, 0)), toKiCADPoint((0, -1)), clearance)
 
     count, cutLength = numberOfCuts(rect.GetHeight() - 2 * R, bridgeWidth, bridgeSpacing)
     for i in range(count):
@@ -112,23 +111,23 @@ def addFrame(board, rect, bridgeWidth, bridgeSpacing, clearance):
         end = start + cutLength
 
         x1, x2 = rect.GetX(), rect.GetX() + rect.GetWidth()
-        addLine(board, VECTOR2I(x1, start), VECTOR2I(x1, end), clearance)
+        addLine(board, toKiCADPoint((x1, start)), toKiCADPoint((x1, end)), clearance)
         if i != 0:
-            addBite(board, VECTOR2I(x1, start), VECTOR2I(0, -1), VECTOR2I(1, 0), clearance)
+            addBite(board, toKiCADPoint((x1, start)), toKiCADPoint((0, -1)), toKiCADPoint((1, 0)), clearance)
         if i != count - 1:
-            addBite(board, VECTOR2I(x1, end), VECTOR2I(0, 1), VECTOR2I(1, 0), clearance)
-        addLine(board, VECTOR2I(x2, start), VECTOR2I(x2, end), clearance)
+            addBite(board, toKiCADPoint((x1, end)), toKiCADPoint((0, 1)), toKiCADPoint((1, 0)), clearance)
+        addLine(board, toKiCADPoint((x2, start)), toKiCADPoint((x2, end)), clearance)
         if i != 0:
-            addBite(board, VECTOR2I(x2, start), VECTOR2I(0, -1), VECTOR2I(-1, 0), clearance)
+            addBite(board, toKiCADPoint((x2, start)), toKiCADPoint((0, -1)), toKiCADPoint((-1, 0)), clearance)
         if i != count - 1:
-            addBite(board, VECTOR2I(x2, end), VECTOR2I(0, 1), VECTOR2I(-1, 0), clearance)
+            addBite(board, toKiCADPoint((x2, end)), toKiCADPoint((0, 1)), toKiCADPoint((-1, 0)), clearance)
 
 def addHole(board, position, radius):
     circle = pcbnew.PCB_SHAPE()
     circle.SetShape(STROKE_T.S_CIRCLE)
-    circle.SetCenter(VECTOR2I(position[0], position[1]))
+    circle.SetCenter(toKiCADPoint((position[0], position[1])))
     # Set 3'oclock point of the circle to set radius
-    circle.SetEnd(VECTOR2I(position[0], position[1]) + VECTOR2I(radius/2, 0))
+    circle.SetEnd(toKiCADPoint((position[0], position[1])) + toKiCADPoint((radius/2, 0)))
 
     circle.SetWidth(radius)
     circle.SetLayer(Layer.F_Paste)
@@ -156,27 +155,27 @@ def addJigFrame(board, jigFrameSize, bridgeWidth=fromMm(2),
 
     for i in range(MOUNTING_HOLES_COUNT):
         x = frameSize.GetX() + OUTER_BORDER / 2 + (i + 1) * (frameSize.GetWidth() - OUTER_BORDER) / (MOUNTING_HOLES_COUNT + 1)
-        addHole(board, VECTOR2I(x, OUTER_BORDER / 2 + frameSize.GetY()), MOUNTING_HOLE_R)
-        addHole(board, VECTOR2I(x, - OUTER_BORDER / 2 +frameSize.GetY() + frameSize.GetHeight()), MOUNTING_HOLE_R)
+        addHole(board, toKiCADPoint((x, OUTER_BORDER / 2 + frameSize.GetY())), MOUNTING_HOLE_R)
+        addHole(board, toKiCADPoint((x, - OUTER_BORDER / 2 +frameSize.GetY() + frameSize.GetHeight())), MOUNTING_HOLE_R)
     for i in range(MOUNTING_HOLES_COUNT):
         y = frameSize.GetY() + OUTER_BORDER / 2 + (i + 1) * (frameSize.GetHeight() - OUTER_BORDER) / (MOUNTING_HOLES_COUNT + 1)
-        addHole(board, VECTOR2I(OUTER_BORDER / 2 + frameSize.GetX(), y), MOUNTING_HOLE_R)
-        addHole(board, VECTOR2I(- OUTER_BORDER / 2 +frameSize.GetX() + frameSize.GetWidth(), y), MOUNTING_HOLE_R)
+        addHole(board, toKiCADPoint((OUTER_BORDER / 2 + frameSize.GetX(), y)), MOUNTING_HOLE_R)
+        addHole(board, toKiCADPoint((- OUTER_BORDER / 2 +frameSize.GetX() + frameSize.GetWidth(), y)), MOUNTING_HOLE_R)
 
     PIN_TOLERANCE = fromMm(0.05)
-    addHole(board, tl(frameSize) + VECTOR2I(OUTER_BORDER / 2, OUTER_BORDER / 2), MOUNTING_HOLE_R + PIN_TOLERANCE)
-    addHole(board, tr(frameSize) + VECTOR2I(-OUTER_BORDER / 2, OUTER_BORDER / 2), MOUNTING_HOLE_R + PIN_TOLERANCE)
-    addHole(board, br(frameSize) + VECTOR2I(-OUTER_BORDER / 2, -OUTER_BORDER / 2), MOUNTING_HOLE_R + PIN_TOLERANCE)
-    addHole(board, bl(frameSize) + VECTOR2I(OUTER_BORDER / 2, -OUTER_BORDER / 2), MOUNTING_HOLE_R + PIN_TOLERANCE)
+    addHole(board, tl(frameSize) + toKiCADPoint((OUTER_BORDER / 2, OUTER_BORDER / 2)), MOUNTING_HOLE_R + PIN_TOLERANCE)
+    addHole(board, tr(frameSize) + toKiCADPoint((-OUTER_BORDER / 2, OUTER_BORDER / 2)), MOUNTING_HOLE_R + PIN_TOLERANCE)
+    addHole(board, br(frameSize) + toKiCADPoint((-OUTER_BORDER / 2, -OUTER_BORDER / 2)), MOUNTING_HOLE_R + PIN_TOLERANCE)
+    addHole(board, bl(frameSize) + toKiCADPoint((OUTER_BORDER / 2, -OUTER_BORDER / 2)), MOUNTING_HOLE_R + PIN_TOLERANCE)
 
-def jigMountingHoles(jigFrameSize, origin=VECTOR2I(0, 0)):
+def jigMountingHoles(jigFrameSize, origin=toKiCADPoint((0, 0))):
     """ Get list of all mounting holes in a jig of given size """
     w, h = jigFrameSize
     holes = [
-        VECTOR2I(0, (w + INNER_BORDER) / 2),
-        VECTOR2I(0, -(w + INNER_BORDER) / 2),
-        VECTOR2I((h + INNER_BORDER) / 2, 0),
-        VECTOR2I(-(h + INNER_BORDER) / 2, 0),
+        toKiCADPoint((0, (w + INNER_BORDER) / 2)),
+        toKiCADPoint((0, -(w + INNER_BORDER) / 2)),
+        toKiCADPoint(((h + INNER_BORDER) / 2, 0)),
+        toKiCADPoint((-(h + INNER_BORDER) / 2, 0)),
     ]
     return [x + origin for x in holes]
 
@@ -305,7 +304,7 @@ def cutoutComponents(board, components):
 def setStencilLayerVisibility(boardName):
     prlPath = os.path.splitext(boardName)[0] + ".kicad_prl"
     try:
-        with open(prlPath) as f:
+        with open(prlPath, encoding="utf-8") as f:
             # We use ordered dict, so we preserve the ordering of the keys and
             # thus, formatting
             prl = json.load(f, object_pairs_hook=OrderedDict)
